// This module deals with converting the notification object into Json format, which might be useful for data exchange
use crate::notification::Notification;
use serde_json::{json, Value}; // json!() macro and Value type

impl Notification {
    // Actions, as json
    pub fn actions_json(&self) -> Value {
        if self.actions().is_empty() {
            json!({}) // Empty JSON object if no actions
        } else {
            serde_json::Value::Object(
                // Wrap the map into a serde_json::Value
                self.actions()
                    .iter()
                    .map(|(id, label)| {
                        (id.clone(), json!(label)) // Create key-value pairs: id -> label
                    })
                    .collect::<serde_json::Map<_, _>>(), // Collect into a serde_json::Map
            )
        }
    }

    // Hints, as json
    pub fn hints_json(&self) -> Value {
        if self.hints().is_empty() {
            json!({}) // Empty JSON object if no hints
        } else {
            serde_json::Value::Object(
                self.hints()
                    .iter()
                    .map(|(key, value)| {
                        (key.clone(), json!(value.to_string())) // Convert hint value to string
                    })
                    .collect::<serde_json::Map<_, _>>(), // Collect into a serde_json::Map
            )
        }
    }

    // The notification as a json object
    pub fn json(&self) -> Value {
        json!({
            "app_name": &self.app_name(),
            "replace_id": &self.replace_id(),
            "icon": &self.icon(),
            "summary": &self.summary(),
            "body": &self.body(),
            "actions": &self.actions_json(),
            "hints": &self.hints_json(),
            "expiration_timeout": self.expir_timeout(),
            "urgency": self.urgency(),
            "default_action": self.default_action().map(|actn| actn.0),
        })
    }
}
