// This module helps get the notification object in a plain text format, which could be printed to the console or a file
use crate::notification::Notification;

impl Notification {
    // Plain string format, useful for printing
    pub fn plain(&self) -> String {
        // Actions
        let actions: String = if self.actions().is_empty() {
            "None".to_string()
        } else {
            self.actions()
                .iter()
                .fold(String::new(), |mut acc, (id, label)| {
                    // The formatted string is appended to the accumulator string, acc
                    acc.push_str(&format!("\n\t{id}: {label}"));
                    acc
                })
        };
        // Default action
        let default_action: String = match self.default_action() {
            None => "None".to_string(),
            Some(actn) => actn.0,
        };
        // Hints
        let hints: String = if self.hints().is_empty() {
            "None".to_string()
        } else {
            self.hints()
                .iter()
                .fold(String::new(), |mut acc, (key, value)| {
                    acc.push_str(&format!("\n\t{}: {}", key, **value));
                    acc
                })
        };

        // Return the notification as a plain string
        let plain_string: String = format!("App Name: {}\nReplace ID: {}\nIcon: {}\nSummary: {}\nBody: {}\nActions: {}\nHints: {}\nExpiration Timeout: {}\nUrgency: {}\nDefault Action: {}",
                        &self.app_name(),
                        &self.replace_id(),
                        &self.icon(),
                        &self.summary(),
                        &self.body(),
                        actions,
                        hints,
                        &self.expir_timeout(),
                        &self.urgency(),
                        default_action,
                    );
        plain_string
    }
}
